@@ -34,6 +34,7 @@ import platform
 import time
 from aip.utils import windows_full_port_name
 from aip.utils import SerialUtils
+from aip.utils import dealGenericOptions
 import serial
 import subprocess
 import posixpath
@@ -50,6 +51,7 @@ class boardCommand(Command):
     summary = "Get basic information of ArduPy board."
 
     def __init__(self, *args, **kw):
+        dealGenericOptions()
         super(boardCommand, self).__init__(*args, **kw)
         self.cmd_opts.add_option(
             '-s', '--scan',
@@ -93,6 +95,7 @@ class boardCommand(Command):
             default=False,
             help='Connect available boards')
         
+        
         self.parser.insert_option_group(0, self.cmd_opts)
     
     def board_halt(self):
@@ -100,8 +103,9 @@ class boardCommand(Command):
         buf = bytearray()
 
         for i in range(0, 3):
+            i
             try:
-                com = self.serial;
+                com = self.serial
                 com.timeout = 1
                 com.writeTimeout = 1
                 com.write(b"\x03")
