@@ -30,47 +30,24 @@ import locale
 import json
 from datetime import date, datetime
 from pathlib import Path
-import urllib.request
-import shutil
 from aip.logger import log
 from pip._internal.exceptions import PipError
 from pip._internal.utils import deprecation
 from pip._internal.cli.autocompletion import autocomplete
 import importlib
 from pip._internal.utils import appdirs
+from aip.parser import parser
 
+if os.name == "nt":
+    	os.system("")
+        
 def main(args=None):
 
      # type: (Optional[List[str]]) -> int
     if args is None:
         args = sys.argv[1:]
 
-    # is update package_seeeduino_ardupy_index.json
-    user_config_dir = str(appdirs.user_config_dir(appname="aip"))
-    if not os.path.exists(user_config_dir):
-        os.makedirs(user_config_dir)
-    today = date.today()
-    user_config_dir_files = os.listdir(user_config_dir)
-    current_package_seeeduino_ardupy = "xxxxx"
-    is_update = True
-    for files in user_config_dir_files:
-        if files[0:30] == "package_seeeduino_ardupy_index":
-            file_data = datetime.strptime(files[31:41], '%Y-%m-%d').date()
-            current_package_seeeduino_ardupy = files
-            if file_data == today:
-                is_update = False
-                break
-    if is_update:
-        log.info("update latest package_seeeduino_ardupy_index.json ...")
-        try:
-            urllib.request.urlretrieve('https://files.seeedstudio.com/ardupy/package_seeeduino_ardupy_index.json',
-                                   str(Path(user_config_dir, "package_seeeduino_ardupy_index_" + today.isoformat() + ".json")))
-            if os.path.exists(str(Path(user_config_dir, current_package_seeeduino_ardupy))):
-                os.remove(str(Path(user_config_dir, current_package_seeeduino_ardupy)))
-        except Exception as e:
-            log.error(e)
-            log.warning("update latest package_seeeduino_ardupy_index.json Failed! Please check you network connction!")
-            sys.exit(1)
+    #parser.update_loacl_board_json()
 
     from aip.command import commands_dict, parse_command
 
